@@ -1,7 +1,6 @@
 #ifndef INTERNAL_TIME_H                                  /*-*-C-*-vi:se ft=c:*/
 #define INTERNAL_TIME_H
 /**
- * @file
  * @author     Ruby developers <ruby-core@ruby-lang.org>
  * @copyright  This  file  is   a  part  of  the   programming  language  Ruby.
  *             Permission  is hereby  granted,  to  either redistribute  and/or
@@ -28,8 +27,8 @@ struct timeval rb_time_timeval(VALUE);
 
 RUBY_SYMBOL_EXPORT_BEGIN
 /* time.c (export) */
-void ruby_reset_leap_second_info(void);
-void ruby_reset_timezone(void);
 RUBY_SYMBOL_EXPORT_END
 
+void ruby_reset_timezone(const char *);
+
 #endif /* INTERNAL_TIME_H */
