@@ -1,13 +1,13 @@
 require_relative "version"
 
 module ErrorHighlight
-  # Identify the code fragment at that a given exception occurred.
+  # Identify the code fragment where a given exception occurred.
   #
   # Options:
   #
   # point_type: :name | :args
-  #   :name (default) points the method/variable name that the exception occurred.
-  #   :args points the arguments of the method call that the exception occurred.
+  #   :name (default) points to the method/variable name where the exception occurred.
+  #   :args points to the arguments of the method call where the exception occurred.
   #
   # backtrace_location: Thread::Backtrace::Location
   #   It locates the code fragment of the given backtrace_location.
@@ -113,7 +113,7 @@ module ErrorHighlight
         snippet = @node.script_lines[lineno - 1 .. last_lineno - 1].join("")
         snippet += "\n" unless snippet.end_with?("\n")
 
-        # It require some work to support Unicode (or multibyte) characters.
+        # It requires some work to support Unicode (or multibyte) characters.
         # Tentatively, we stop highlighting if the code snippet has non-ascii characters.
         # See https://github.com/ruby/error_highlight/issues/4
         raise NonAscii unless snippet.ascii_only?
@@ -122,56 +122,51 @@ module ErrorHighlight
       end
     end
 
-    OPT_GETCONSTANT_PATH = (RUBY_VERSION.split(".").map {|s| s.to_i } <=> [3, 2]) >= 0
-    private_constant :OPT_GETCONSTANT_PATH
-
     def spot
       return nil unless @node
 
-      if OPT_GETCONSTANT_PATH
-        # In Ruby 3.2 or later, a nested constant access (like `Foo::Bar::Baz`)
-        # is compiled to one instruction (opt_getconstant_path).
-        # @node points to the node of the whole `Foo::Bar::Baz` even if `Foo`
-        # or `Foo::Bar` causes NameError.
-        # So we try to spot the sub-node that causes the NameError by using
-        # `NameError#name`.
-        case @node.type
-        when :COLON2
-          subnodes = []
-          node = @node
-          while node.type == :COLON2
-            node2, const = node.children
-            subnodes << node if const == @name
-            node = node2
-          end
-          if node.type == :CONST || node.type == :COLON3
-            if node.children.first == @name
-              subnodes << node
-            end
-
-            # If we found only one sub-node whose name is equal to @name, use it
-            return nil if subnodes.size != 1
-            @node = subnodes.first
-          else
-            # Do nothing; opt_getconstant_path is used only when the const base is
-            # NODE_CONST (`Foo`) or NODE_COLON3 (`::Foo`)
-          end
-        when :constant_path_node
-          subnodes = []
-          node = @node
-
-          begin
-            subnodes << node if node.name == @name
-          end while (node = node.parent).is_a?(Prism::ConstantPathNode)
-
-          if node.is_a?(Prism::ConstantReadNode) && node.name == @name
+      # In Ruby 3.2 or later, a nested constant access (like `Foo::Bar::Baz`)
+      # is compiled to one instruction (opt_getconstant_path).
+      # @node points to the node of the whole `Foo::Bar::Baz` even if `Foo`
+      # or `Foo::Bar` causes NameError.
+      # So we try to spot the sub-node that causes the NameError by using
+      # `NameError#name`.
+      case @node.type
+      when :COLON2
+        subnodes = []
+        node = @node
+        while node.type == :COLON2
+          node2, const = node.children
+          subnodes << node if const == @name
+          node = node2
+        end
+        if node.type == :CONST || node.type == :COLON3
+          if node.children.first == @name
             subnodes << node
           end
 
           # If we found only one sub-node whose name is equal to @name, use it
           return nil if subnodes.size != 1
           @node = subnodes.first
+        else
+          # Do nothing; opt_getconstant_path is used only when the const base is
+          # NODE_CONST (`Foo`) or NODE_COLON3 (`::Foo`)
         end
+      when :constant_path_node
+        subnodes = []
+        node = @node
+
+        begin
+          subnodes << node if node.name == @name
+        end while (node = node.parent).is_a?(Prism::ConstantPathNode)
+
+        if node.is_a?(Prism::ConstantReadNode) && node.name == @name
+          subnodes << node
+        end
+
+        # If we found only one sub-node whose name is equal to @name, use it
+        return nil if subnodes.size != 1
+        @node = subnodes.first
       end
 
       case @node.type
@@ -239,6 +234,20 @@ module ErrorHighlight
       when :OP_CDECL
         spot_op_cdecl
 
+      when :DEFN
+        raise NotImplementedError if @point_type != :name
+        spot_defn
+
+      when :DEFS
+        raise NotImplementedError if @point_type != :name
+        spot_defs
+
+      when :LAMBDA
+        spot_lambda
+
+      when :ITER
+        spot_iter
+
       when :call_node
         case @point_type
         when :name
@@ -280,6 +289,30 @@ module ErrorHighlight
       when :constant_path_operator_write_node
         prism_spot_constant_path_operator_write
 
+      when :def_node
+        case @point_type
+        when :name
+          prism_spot_def_for_name
+        when :args
+          raise NotImplementedError
+        end
+
+      when :lambda_node
+        case @point_type
+        when :name
+          prism_spot_lambda_for_name
+        when :args
+          raise NotImplementedError
+        end
+
+      when :block_node
+        case @point_type
+        when :name
+          prism_spot_block_for_name
+        when :args
+          raise NotImplementedError
+        end
+
       end
 
       if @snippet && @beg_column && @end_column && @beg_column < @end_column
@@ -344,6 +377,7 @@ module ErrorHighlight
         end
       elsif mid.to_s =~ /\A\W+\z/ && lines.match(/\G\s*(#{ Regexp.quote(mid) })=.*\n/, nd_recv.last_column)
         @snippet = $` + $&
+        @beg_lineno = @end_lineno = lineno
         @beg_column = $~.begin(1)
         @end_column = $~.end(1)
       end
@@ -470,7 +504,6 @@ module ErrorHighlight
     def spot_fcall_for_args
       _mid, nd_args = @node.children
       if nd_args && nd_args.first_lineno == nd_args.last_lineno
-        # binary operator
         fetch_line(nd_args.first_lineno)
         @beg_column = nd_args.first_column
         @end_column = nd_args.last_column
@@ -582,8 +615,9 @@ module ErrorHighlight
         @beg_column = nd_parent.last_column
         @end_column = @node.last_column
       else
-        @snippet = @fetch[@node.last_lineno]
+        fetch_line(@node.last_lineno)
         if @snippet[...@node.last_column].match(/#{ Regexp.quote(const) }\z/)
+          @beg_lineno = @end_lineno = @node.last_lineno
           @beg_column = $~.begin(0)
           @end_column = $~.end(0)
         end
@@ -597,7 +631,7 @@ module ErrorHighlight
       nd_lhs, op, _nd_rhs = @node.children
       *nd_parent_lhs, _const = nd_lhs.children
       if @name == op
-        @snippet = @fetch[nd_lhs.last_lineno]
+        fetch_line(nd_lhs.last_lineno)
         if @snippet.match(/\G\s*(#{ Regexp.quote(op) })=/, nd_lhs.last_column)
           @beg_column = $~.begin(1)
           @end_column = $~.end(1)
@@ -607,18 +641,67 @@ module ErrorHighlight
         @end_column = nd_lhs.last_column
         if nd_parent_lhs.empty? # example: ::C += 1
           if nd_lhs.first_lineno == nd_lhs.last_lineno
-            @snippet = @fetch[nd_lhs.last_lineno]
+            fetch_line(nd_lhs.last_lineno)
             @beg_column = nd_lhs.first_column
           end
         else # example: Foo::Bar::C += 1
           if nd_parent_lhs.last.last_lineno == nd_lhs.last_lineno
-            @snippet = @fetch[nd_lhs.last_lineno]
+            fetch_line(nd_lhs.last_lineno)
             @beg_column = nd_parent_lhs.last.last_column
           end
         end
       end
     end
 
+    # Example:
+    #   def bar; end
+    #       ^^^
+    def spot_defn
+      mid, = @node.children
+      fetch_line(@node.first_lineno)
+      if @snippet.match(/\Gdef\s+(#{ Regexp.quote(mid) }\b)/, @node.first_column)
+        @beg_column = $~.begin(1)
+        @end_column = $~.end(1)
+      end
+    end
+
+    # Example:
+    #   def Foo.bar; end
+    #          ^^^^
+    def spot_defs
+      nd_recv, mid, = @node.children
+      fetch_line(nd_recv.last_lineno)
+      if @snippet.match(/\G\s*(\.\s*#{ Regexp.quote(mid) }\b)/, nd_recv.last_column)
+        @beg_column = $~.begin(1)
+        @end_column = $~.end(1)
+      end
+    end
+
+    # Example:
+    #   -> { ... }
+    #   ^^
+    def spot_lambda
+      fetch_line(@node.first_lineno)
+      if @snippet.match(/\G->/, @node.first_column)
+        @beg_column = $~.begin(0)
+        @end_column = $~.end(0)
+      end
+    end
+
+    # Example:
+    #   lambda { ... }
+    #          ^
+    #   define_method :foo do
+    #                      ^^
+    def spot_iter
+      _nd_fcall, nd_scope = @node.children
+      fetch_line(nd_scope.first_lineno)
+      if @snippet.match(/\G(?:do\b|\{)/, nd_scope.first_column)
+        @beg_column = $~.begin(0)
+        @end_column = $~.end(0)
+      end
+    end
+
     def fetch_line(lineno)
       @beg_lineno = @end_lineno = lineno
       @snippet = @fetch[lineno]
@@ -824,6 +907,31 @@ module ErrorHighlight
         prism_location(@node.binary_operator_loc.chop)
       end
     end
+
+    # Example:
+    #   def foo()
+    #       ^^^
+    def prism_spot_def_for_name
+      location = @node.name_loc
+      location = @node.operator_loc.join(location) if @node.operator_loc
+      prism_location(location)
+    end
+
+    # Example:
+    #   -> x, y { }
+    #   ^^
+    def prism_spot_lambda_for_name
+      prism_location(@node.operator_loc)
+    end
+
+    # Example:
+    #   lambda { }
+    #          ^
+    #   define_method :foo do |x, y|
+    #                      ^
+    def prism_spot_block_for_name
+      prism_location(@node.opening_loc)
+    end
   end
 
   private_constant :Spotter
