@@ -1,8 +1,6 @@
 # frozen_string_literal: true
 
-require_relative "compact_index"
-
-Artifice.deactivate
+require_relative "helpers/compact_index"
 
 class CompactIndexNoGem < CompactIndexAPI
   get "/gems/:id" do
@@ -10,4 +8,6 @@ class CompactIndexNoGem < CompactIndexAPI
   end
 end
 
+require_relative "helpers/artifice"
+
 Artifice.activate_with(CompactIndexNoGem)
