@@ -16,14 +16,19 @@ describe "File.mtime" do
   end
 
   platform_is :linux, :windows do
-    it "returns the modification Time of the file with microseconds" do
-      supports_subseconds = Integer(`stat -c%y '#{__FILE__}'`[/\.(\d+)/, 1], 10)
-      if supports_subseconds != 0
-        expected_time = Time.at(Time.now.to_i + 0.123456)
-        File.utime 0, expected_time, @filename
-        File.mtime(@filename).usec.should == expected_time.usec
-      else
-        File.mtime(__FILE__).usec.should == 0
+    unless ENV.key?('TRAVIS') # https://bugs.ruby-lang.org/issues/17926
+      it "returns the modification Time of the file with microseconds" do
+        supports_subseconds = Integer(`stat -c%y '#{__FILE__}'`[/\.(\d{1,6})/, 1], 10)
+        if supports_subseconds != 0
+          expected_time = Time.at(Time.now.to_i + 0.123456)
+          File.utime 0, expected_time, @filename
+          File.mtime(@filename).usec.should == expected_time.usec
+        else
+          File.mtime(__FILE__).usec.should == 0
+        end
+      rescue Errno::ENOENT => e
+        # Windows don't have stat command.
+        skip e.message
       end
     end
   end
