@@ -3,7 +3,7 @@ require_relative 'fixtures/classes'
 
 describe "StringIO#binmode" do
   it "returns self" do
-    io = StringIO.new("example")
+    io = StringIO.new(+"example")
     io.binmode.should equal(io)
   end
 
