@@ -1,7 +1,5 @@
 require_relative "../../../spec_helper"
 platform_is :windows do
-  verbose, $VERBOSE = $VERBOSE, nil
-
   require_relative '../fixtures/classes'
 
   describe "WIN32OLE.const_load when passed Shell.Application OLE object" do
@@ -32,6 +30,4 @@ platform_is :windows do
     end
   end
 
-ensure
-  $VERBOSE = verbose
 end
