@@ -1,7 +1,5 @@
 require_relative "../../../spec_helper"
 platform_is :windows do
-  verbose, $VERBOSE = $VERBOSE, nil
-
   require_relative '../fixtures/classes'
 
   describe "WIN32OLESpecs.new_ole" do
@@ -19,12 +17,10 @@ platform_is :windows do
       -> { WIN32OLESpecs.new_ole(42) }.should raise_error( TypeError )
     end
 
-    it "raises WIN32OLERuntimeError if invalid string is given" do
-      -> { WIN32OLE.new('foo') }.should raise_error( WIN32OLERuntimeError )
+    it "raises WIN32OLE::RuntimeError if invalid string is given" do
+      -> { WIN32OLE.new('foo') }.should raise_error( WIN32OLE::RuntimeError )
     end
 
   end
 
-ensure
-  $VERBOSE = verbose
 end
