@@ -1,7 +1,5 @@
 require_relative "../../../spec_helper"
 platform_is :windows do
-  verbose, $VERBOSE = $VERBOSE, nil
-
   require_relative '../fixtures/classes'
 
   describe "WIN32OLE#ole_methods" do
@@ -13,14 +11,12 @@ platform_is :windows do
       -> { @dict.ole_methods(1) }.should raise_error ArgumentError
     end
 
-    it "returns an array of WIN32OLE_METHODs" do
-      @dict.ole_methods.all? { |m| m.kind_of? WIN32OLE_METHOD }.should be_true
+    it "returns an array of WIN32OLE::Methods" do
+      @dict.ole_methods.all? { |m| m.kind_of? WIN32OLE::Method }.should be_true
     end
 
     it "contains a 'AddRef' method for Scripting Dictionary" do
       @dict.ole_methods.map { |m| m.name }.include?('AddRef').should be_true
     end
   end
-ensure
-  $VERBOSE = verbose
 end
