@@ -1,13 +1,11 @@
 require_relative "../../../spec_helper"
 platform_is :windows do
-  verbose, $VERBOSE = $VERBOSE, nil
-
   require 'win32ole'
 
-  describe "WIN32OLE_METHOD#invoke_kind" do
+  describe "WIN32OLE::Method#invoke_kind" do
     before :each do
-      ole_type = WIN32OLE_TYPE.new("Microsoft Scripting Runtime", "File")
-      @m_file_name = WIN32OLE_METHOD.new(ole_type, "name")
+      ole_type = WIN32OLE::Type.new("Microsoft Scripting Runtime", "File")
+      @m_file_name = WIN32OLE::Method.new(ole_type, "name")
     end
 
     it "raises ArgumentError if argument is given" do
@@ -20,6 +18,4 @@ platform_is :windows do
 
   end
 
-ensure
-  $VERBOSE = verbose
 end
