@@ -1,9 +1,10 @@
+require_relative "../../../spec_helper"
 platform_is :windows do
   require 'win32ole'
 
-  describe "WIN32OLE_TYPE#minor_version for Shell Controls" do
+  describe "WIN32OLE::Type#minor_version for Shell Controls" do
     before :each do
-      @ole_type = WIN32OLE_TYPE.new("Microsoft Shell Controls And Automation", "Shell")
+      @ole_type = WIN32OLE::Type.new("Microsoft Shell Controls And Automation", "Shell")
     end
 
     after :each do
