@@ -32,7 +32,7 @@ class TestObjSpace < Test::Unit::TestCase
     a = "a" * GC::INTERNAL_CONSTANTS[:RVARGC_MAX_ALLOCATE_SIZE]
     b = a.dup
     c = nil
-    ObjectSpace.each_object(String) {|x| break c = x if x == a and x.frozen?}
+    ObjectSpace.each_object(String) {|x| break c = x if a == x and x.frozen?}
     rv_size = GC::INTERNAL_CONSTANTS[:BASE_SLOT_SIZE]
     assert_equal([rv_size, rv_size, a.length + 1 + rv_size], [a, b, c].map {|x| ObjectSpace.memsize_of(x)})
   end
@@ -203,8 +203,9 @@ class TestObjSpace < Test::Unit::TestCase
       assert_equal(line1,    ObjectSpace.allocation_sourceline(o1))
       assert_equal(__FILE__, ObjectSpace.allocation_sourcefile(o1))
       assert_equal(c1,       ObjectSpace.allocation_generation(o1))
-      assert_equal(Class.name, ObjectSpace.allocation_class_path(o1))
-      assert_equal(:new,       ObjectSpace.allocation_method_id(o1))
+      # These assertions fail under coverage measurement: https://bugs.ruby-lang.org/issues/21298
+      #assert_equal(self.class.name, ObjectSpace.allocation_class_path(o1))
+      #assert_equal(__method__,       ObjectSpace.allocation_method_id(o1))
 
       assert_equal(__FILE__, ObjectSpace.allocation_sourcefile(o2))
       assert_equal(line2,    ObjectSpace.allocation_sourceline(o2))
@@ -287,6 +288,33 @@ class TestObjSpace < Test::Unit::TestCase
     assert true # success
   end
 
+  def test_trace_object_allocations_with_other_tracepoint
+    # Test that ObjectSpace.trace_object_allocations isn't changed by changes
+    # to another tracepoint
+    line_tp = TracePoint.new(:line) { }
+
+    ObjectSpace.trace_object_allocations_start
+
+    obj1 = Object.new; line1 = __LINE__
+    assert_equal __FILE__, ObjectSpace.allocation_sourcefile(obj1)
+    assert_equal line1, ObjectSpace.allocation_sourceline(obj1)
+
+    line_tp.enable
+
+    obj2 = Object.new; line2 = __LINE__
+    assert_equal __FILE__, ObjectSpace.allocation_sourcefile(obj2)
+    assert_equal line2, ObjectSpace.allocation_sourceline(obj2)
+
+    line_tp.disable
+
+    obj3 = Object.new; line3 = __LINE__
+    assert_equal __FILE__, ObjectSpace.allocation_sourcefile(obj3)
+    assert_equal line3, ObjectSpace.allocation_sourceline(obj3)
+  ensure
+    ObjectSpace.trace_object_allocations_stop
+    ObjectSpace.trace_object_allocations_clear
+  end
+
   def test_trace_object_allocations_compaction
     omit "compaction is not supported on this platform" unless GC.respond_to?(:compact)
 
@@ -308,7 +336,7 @@ class TestObjSpace < Test::Unit::TestCase
   def test_trace_object_allocations_compaction_freed_pages
     omit "compaction is not supported on this platform" unless GC.respond_to?(:compact)
 
-    assert_normal_exit(<<~RUBY)
+    assert_normal_exit(<<~RUBY, timeout: 60)
       require "objspace"
 
       objs = []
@@ -332,11 +360,25 @@ class TestObjSpace < Test::Unit::TestCase
     # Ensure that the fstring is promoted to old generation
     4.times { GC.start }
     info = ObjectSpace.dump("foo".freeze)
-    assert_match(/"wb_protected":true, "old":true/, info)
+    assert_include(info, '"wb_protected":true')
+    assert_include(info, '"age":3')
+    assert_include(info, '"old":true')
     assert_match(/"fstring":true/, info)
     JSON.parse(info) if defined?(JSON)
   end
 
+  def test_dump_flag_age
+    EnvUtil.without_gc do
+      o = Object.new
+
+      assert_include(ObjectSpace.dump(o), '"age":0')
+
+      GC.start
+
+      assert_include(ObjectSpace.dump(o), '"age":1')
+    end
+  end
+
   if defined?(RubyVM::Shape)
     class TooComplex; end
 
@@ -486,6 +528,20 @@ class TestObjSpace < Test::Unit::TestCase
     assert_match(/"value":"foobar\h+"/, dump)
   end
 
+  def test_dump_outputs_object_id
+    obj = Object.new
+
+    # Doesn't output object_id when it has not been seen
+    dump = ObjectSpace.dump(obj)
+    assert_not_include(dump, "\"object_id\"")
+
+    id = obj.object_id
+
+    # Outputs object_id when it has been seen
+    dump = ObjectSpace.dump(obj)
+    assert_include(dump, "\"object_id\":#{id}")
+  end
+
   def test_dump_includes_imemo_type
     assert_in_out_err(%w[-robjspace], "#{<<-"begin;"}\n#{<<-'end;'}") do |output, error|
       begin;
@@ -771,6 +827,27 @@ class TestObjSpace < Test::Unit::TestCase
     end
   end
 
+  def test_dump_all_with_ractors
+    assert_ractor("#{<<-"begin;"}#{<<-'end;'}")
+    begin;
+      require "objspace"
+      require "tempfile"
+      require "json"
+      rs = 4.times.map do
+        Ractor.new do
+          Tempfile.create do |f|
+            ObjectSpace.dump_all(output: f)
+            f.close
+            File.readlines(f.path).each do |line|
+              JSON.parse(line)
+            end
+          end
+        end
+      end
+      rs.each(&:join)
+    end;
+  end
+
   def test_dump_uninitialized_file
     assert_in_out_err(%[-robjspace], <<-RUBY) do |(output), (error)|
       puts ObjectSpace.dump(File.allocate)
@@ -949,6 +1026,13 @@ class TestObjSpace < Test::Unit::TestCase
     assert_equal class_name, JSON.parse(json)["name"]
   end
 
+  def test_dump_include_shareable
+    omit 'Not provided by mmtk' if RUBY_DESCRIPTION.include?("+GC[mmtk]")
+
+    assert_include(ObjectSpace.dump(ENV), '"shareable":true')
+    assert_not_include(ObjectSpace.dump([]), '"shareable":true')
+  end
+
   def test_utf8_method_names
     name = "utf8_❨╯°□°❩╯︵┻━┻"
     obj = ObjectSpace.trace_object_allocations do
