@@ -1,6 +1,5 @@
 # frozen_string_literal: false
 require 'test/unit'
-require '-test-/time'
 
 class TestTimeTZ < Test::Unit::TestCase
   has_right_tz = true
@@ -115,7 +114,7 @@ class TestTimeTZ < Test::Unit::TestCase
     t = with_tz("America/Los_Angeles") {
       Time.local(2000, 1, 1)
     }
-    skip "force_tz_test is false on this environment" unless t
+    omit "force_tz_test is false on this environment" unless t
     z1 = t.zone
     z2 = with_tz(tz="Asia/Singapore") {
       t.localtime.zone
@@ -267,6 +266,8 @@ class TestTimeTZ < Test::Unit::TestCase
     assert_predicate(Time.new(2019, 1, 1, 0, 0, 0, "UTC"), :utc?)
     assert_predicate(Time.new(2019, 1, 1, 0, 0, 0, "utc"), :utc?)
     assert_predicate(Time.new(2019, 1, 1, 0, 0, 0, "Z"), :utc?)
+    assert_predicate(Time.new(2019, 1, 1, 0, 0, 0, "-00:00"), :utc?)
+    assert_not_predicate(Time.new(2019, 1, 1, 0, 0, 0, "+00:00"), :utc?)
   end
 
   def test_military_names
@@ -615,6 +616,13 @@ module TestTimeTZ::WithTZ
     assert_equal(time_class.utc(2018, 9, 1, 12+h, m, 0).to_i, t.to_i)
     assert_equal(6, t.wday)
     assert_equal(244, t.yday)
+    assert_equal(t, time_class.new(2018, 9, 1, 12, in: tzarg))
+    assert_raise(ArgumentError) {time_class.new(2018, 9, 1, 12, 0, 0, tzarg, in: tzarg)}
+  end
+
+  def subtest_hour24(time_class, tz, tzarg, tzname, abbr, utc_offset)
+    t = time_class.new(2000, 1, 1, 24, 0, 0, tzarg)
+    assert_equal([0, 0, 0, 2, 1, 2000], [t.sec, t.min, t.hour, t.mday, t.mon, t.year])
   end
 
   def subtest_now(time_class, tz, tzarg, tzname, abbr, utc_offset)
@@ -686,6 +694,13 @@ module TestTimeTZ::WithTZ
     assert_equal(t.dst?, t2.dst?)
   end
 
+  def subtest_fractional_second(time_class, tz, tzarg, tzname, abbr, utc_offset)
+    t = time_class.new(2024, 1, 1, 23, 59, 59.9r, tzarg)
+    assert_equal(utc_offset[t.dst? ? 1 : 0], t.utc_offset)
+    t = time_class.new(2024, 7, 1, 23, 59, 59.9r, tzarg)
+    assert_equal(utc_offset[t.dst? ? 1 : 0], t.utc_offset)
+  end
+
   def test_invalid_zone
     make_timezone("INVALID", "INV", 0)
   rescue => e
@@ -710,6 +725,7 @@ module TestTimeTZ::WithTZ
     "Asia/Tokyo" => ["JST", +9*3600],
     "America/Los_Angeles" => ["PST", -8*3600, "PDT", -7*3600],
     "Africa/Ndjamena" => ["WAT", +1*3600],
+    "Etc/UTC" => ["UTC", 0],
   }
 
   def make_timezone(tzname, abbr, utc_offset, abbr2 = nil, utc_offset2 = nil)
