@@ -91,20 +91,20 @@ class TestSocket < Test::Unit::TestCase
 
   def test_getaddrinfo
     # This should not send a DNS query because AF_UNIX.
-    assert_raise(SocketError) { Socket.getaddrinfo("www.kame.net", 80, "AF_UNIX") }
+    assert_raise(Socket::ResolutionError) { Socket.getaddrinfo("www.kame.net", 80, "AF_UNIX") }
   end
 
   def test_getaddrinfo_raises_no_errors_on_port_argument_of_0 # [ruby-core:29427]
     assert_nothing_raised('[ruby-core:29427]'){ Socket.getaddrinfo('localhost', 0, Socket::AF_INET, Socket::SOCK_STREAM, nil, Socket::AI_CANONNAME) }
     assert_nothing_raised('[ruby-core:29427]'){ Socket.getaddrinfo('localhost', '0', Socket::AF_INET, Socket::SOCK_STREAM, nil, Socket::AI_CANONNAME) }
     assert_nothing_raised('[ruby-core:29427]'){ Socket.getaddrinfo('localhost', '00', Socket::AF_INET, Socket::SOCK_STREAM, nil, Socket::AI_CANONNAME) }
-    assert_raise(SocketError, '[ruby-core:29427]'){ Socket.getaddrinfo(nil, nil, Socket::AF_INET, Socket::SOCK_STREAM, nil, Socket::AI_CANONNAME) }
+    assert_raise(Socket::ResolutionError, '[ruby-core:29427]'){ Socket.getaddrinfo(nil, nil, Socket::AF_INET, Socket::SOCK_STREAM, nil, Socket::AI_CANONNAME) }
     assert_nothing_raised('[ruby-core:29427]'){ TCPServer.open('localhost', 0) {} }
   end
 
 
   def test_getnameinfo
-    assert_raise(SocketError) { Socket.getnameinfo(["AF_UNIX", 80, "0.0.0.0"]) }
+    assert_raise(Socket::ResolutionError) { Socket.getnameinfo(["AF_UNIX", 80, "0.0.0.0"]) }
     assert_raise(ArgumentError) {Socket.getnameinfo(["AF_INET", "http\0", "example.net"])}
     assert_raise(ArgumentError) {Socket.getnameinfo(["AF_INET", "http", "example.net\0"])}
   end
@@ -163,11 +163,22 @@ class TestSocket < Test::Unit::TestCase
   def random_port
     # IANA suggests dynamic port for 49152 to 65535
     # http://www.iana.org/assignments/port-numbers
-    49152 + rand(65535-49152+1)
+    case RUBY_PLATFORM
+    when /mingw|mswin/
+      rand(50000..65535)
+    else
+      rand(49152..65535)
+    end
   end
 
   def errors_addrinuse
-    [Errno::EADDRINUSE]
+    errs = [Errno::EADDRINUSE]
+    # Windows can fail with "Errno::EACCES: Permission denied - bind(2) for 0.0.0.0:49721"
+    # or "Test::Unit::ProxyError: Permission denied - bind(2) for 0.0.0.0:55333"
+    if /mswin|mingw/ =~ RUBY_PLATFORM
+      errs += [Errno::EACCES, Test::Unit::ProxyError]
+    end
+    errs
   end
 
   def test_tcp_server_sockets
@@ -332,10 +343,14 @@ class TestSocket < Test::Unit::TestCase
   end
 
   def test_udp_server
+    # http://rubyci.s3.amazonaws.com/rhel_zlinux/ruby-master/log/20230312T023302Z.fail.html.gz
+    # Errno::EHOSTUNREACH: No route to host - recvmsg(2)
+    omit if 'rhel_zlinux' == ENV['RUBYCI_NICKNAME']
+
     begin
       ifaddrs = Socket.getifaddrs
     rescue NotImplementedError
-      skip "Socket.getifaddrs not implemented"
+      omit "Socket.getifaddrs not implemented"
     end
 
     ifconfig = nil
@@ -429,16 +444,18 @@ class TestSocket < Test::Unit::TestCase
         }
       rescue NotImplementedError, Errno::ENOSYS
         skipped = true
-        skip "need sendmsg and recvmsg: #{$!}"
+        omit "need sendmsg and recvmsg: #{$!}"
+      rescue RuntimeError
+        skipped = true
+        omit "UDP server is no response: #{$!}"
       ensure
         if th
-          if skipped
-            Thread.kill th unless th.join(10)
-          else
+          unless skipped
             Addrinfo.udp("127.0.0.1", port).connect {|s| s.sendmsg "exit" }
-            unless th.join(10)
-              Thread.kill th
-              th.join(10)
+          end
+          unless th.join(10)
+            th.kill.join(10)
+            unless skipped
               raise "thread killed"
             end
           end
@@ -475,13 +492,14 @@ class TestSocket < Test::Unit::TestCase
         end while IO.select([r], nil, nil, 0.1).nil?
         n
       end
-      timeout = (defined?(RubyVM::MJIT) && RubyVM::MJIT.enabled? ? 120 : 30) # for --jit-wait
+      timeout = 30
       assert_equal([[s1],[],[]], IO.select([s1], nil, nil, timeout))
       msg, _, _, stamp = s1.recvmsg
       assert_equal("a", msg)
       assert(stamp.cmsg_is?(:SOCKET, type))
       w.close # stop th
       n = th.value
+      th = nil
       n > 1 and
         warn "UDP packet loss for #{type} over loopback, #{n} tries needed"
       t2 = Time.now.strftime("%Y-%m-%d")
@@ -490,11 +508,15 @@ class TestSocket < Test::Unit::TestCase
       t = stamp.timestamp
       assert_match(pat, t.strftime("%Y-%m-%d"))
       stamp
+    ensure
+      if th and !th.join(10)
+        th.kill.join(10)
+      end
     end
   end
 
   def test_timestamp
-    return if /linux|freebsd|netbsd|openbsd|solaris|darwin/ !~ RUBY_PLATFORM
+    return if /linux|freebsd|netbsd|openbsd|darwin/ !~ RUBY_PLATFORM
     return if !defined?(Socket::AncillaryData) || !defined?(Socket::SO_TIMESTAMP)
     t1 = Time.now.strftime("%Y-%m-%d")
     stamp = nil
@@ -572,7 +594,7 @@ class TestSocket < Test::Unit::TestCase
   ensure
     serv_thread.value.close
     server.close
-  end
+  end unless RUBY_PLATFORM.include?("freebsd")
 
   def test_connect_timeout
     host = "127.0.0.1"
@@ -751,4 +773,330 @@ class TestSocket < Test::Unit::TestCase
     s2.close
   end
 
+  def test_resolurion_error_error_code
+    begin
+      Socket.getaddrinfo("example.com", 80, "AF_UNIX")
+    rescue => e
+      assert_include([Socket::EAI_FAMILY, Socket::EAI_FAIL], e.error_code)
+    end
+  end
+
+  def test_tcp_socket_v6_hostname_resolved_earlier
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      begin
+        # Verify that "localhost" can be resolved to an IPv6 address
+        Socket.getaddrinfo("localhost", 0, Socket::AF_INET6)
+        server = TCPServer.new("::1", 0)
+      rescue Socket::ResolutionError, Errno::EADDRNOTAVAIL # IPv6 is not supported
+        return
+      end
+
+      _, port, = server.addr
+      server_thread = Thread.new { server.accept }
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+        case family
+        when Socket::AF_INET6 then [Addrinfo.tcp("::1", port)]
+        when Socket::AF_INET then sleep(10); [Addrinfo.tcp("127.0.0.1", port)]
+        end
+      end
+
+      socket = Socket.tcp("localhost", port)
+      assert_true(socket.remote_address.ipv6?)
+    ensure
+      server_thread&.value&.close
+      server&.close
+      socket&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_v4_hostname_resolved_earlier
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      server = TCPServer.new("127.0.0.1", 0)
+      _, port, = server.addr
+      server_thread = Thread.new { server.accept }
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+        case family
+        when Socket::AF_INET6 then sleep(10); [Addrinfo.tcp("::1", port)]
+        when Socket::AF_INET then [Addrinfo.tcp("127.0.0.1", port)]
+        end
+      end
+
+      socket = Socket.tcp("localhost", port)
+      assert_true(socket.remote_address.ipv4?)
+    ensure
+      server_thread&.value&.close
+      server&.close
+      socket&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_v6_hostname_resolved_in_resolution_delay
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      begin
+        # Verify that "localhost" can be resolved to an IPv6 address
+        Socket.getaddrinfo("localhost", 0, Socket::AF_INET6)
+        server = TCPServer.new("::1", 0)
+      rescue Socket::ResolutionError, Errno::EADDRNOTAVAIL # IPv6 is not supported
+        return
+      end
+
+      _, port, = server.addr
+      server_thread = Thread.new { server.accept }
+
+      delay_time = 0.025 # Socket::RESOLUTION_DELAY (private) is 0.05
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+        case family
+        when Socket::AF_INET6 then sleep(delay_time); [Addrinfo.tcp("::1", port)]
+        when Socket::AF_INET then [Addrinfo.tcp("127.0.0.1", port)]
+        end
+      end
+
+      socket = Socket.tcp("localhost", port)
+      assert_true(socket.remote_address.ipv6?)
+    ensure
+      server_thread&.value&.close
+      server&.close
+      socket&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_v6_hostname_resolved_earlier_and_v6_server_is_not_listening
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      ipv4_address = "127.0.0.1"
+      server = Socket.new(Socket::AF_INET, :STREAM)
+      server.bind(Socket.pack_sockaddr_in(0, ipv4_address))
+      port = server.connect_address.ip_port
+      server_thread = Thread.new { server.listen(1); server.accept }
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+        case family
+        when Socket::AF_INET6 then [Addrinfo.tcp("::1", port)]
+        when Socket::AF_INET then sleep(0.001); [Addrinfo.tcp(ipv4_address, port)]
+        end
+      end
+
+      socket = Socket.tcp("localhost", port)
+      assert_equal(ipv4_address, socket.remote_address.ip_address)
+    ensure
+      accepted, _ = server_thread&.value
+      accepted&.close
+      server&.close
+      socket&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_resolv_timeout
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      server = TCPServer.new("localhost", 0)
+      _, port, = server.addr
+
+      Addrinfo.define_singleton_method(:getaddrinfo) { |*_| sleep }
+
+      assert_raise(IO::TimeoutError) do
+        Socket.tcp("localhost", port, resolv_timeout: 0.01)
+      end
+    ensure
+      server&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_resolv_timeout_with_connection_failure
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    server = TCPServer.new("127.0.0.1", 12345)
+    _, port, = server.addr
+
+    Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+      if family == Socket::AF_INET6
+        sleep
+      else
+        [Addrinfo.tcp("127.0.0.1", port)]
+      end
+    end
+
+    server.close
+
+    assert_raise(IO::TimeoutError) do
+      Socket.tcp("localhost", port, resolv_timeout: 0.01)
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_open_timeout
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+      if family == Socket::AF_INET6
+        sleep
+      else
+        [Addrinfo.tcp("127.0.0.1", 12345)]
+      end
+    end
+
+    assert_raise(IO::TimeoutError) do
+      Socket.tcp("localhost", 12345, open_timeout: 0.01)
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_open_timeout_with_other_timeouts
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    assert_raise(ArgumentError) do
+      Socket.tcp("localhost", 12345, open_timeout: 0.01, resolv_timout: 0.01)
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_one_hostname_resolution_succeeded_at_least
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      begin
+        # Verify that "localhost" can be resolved to an IPv6 address
+        Socket.getaddrinfo("localhost", 0, Socket::AF_INET6)
+        server = TCPServer.new("::1", 0)
+      rescue Socket::ResolutionError, Errno::EADDRNOTAVAIL # IPv6 is not supported
+        return
+      end
+
+      _, port, = server.addr
+      server_thread = Thread.new { server.accept }
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+        case family
+        when Socket::AF_INET6 then [Addrinfo.tcp("::1", port)]
+        when Socket::AF_INET then sleep(0.001); raise SocketError
+        end
+      end
+
+      socket = nil
+
+      assert_nothing_raised do
+        socket = Socket.tcp("localhost", port)
+      end
+    ensure
+      server_thread&.value&.close
+      server&.close
+      socket&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_all_hostname_resolution_failed
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      server = TCPServer.new("localhost", 0)
+      _, port, = server.addr
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+        case family
+        when Socket::AF_INET6 then raise SocketError
+        when Socket::AF_INET then sleep(0.01); raise SocketError, "Last hostname resolution error"
+        end
+      end
+
+      assert_raise_with_message(SocketError, "Last hostname resolution error") do
+        Socket.tcp("localhost", port)
+      end
+    ensure
+      server&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_hostname_resolution_failed_after_connection_failure
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    server = TCPServer.new("127.0.0.1", 0)
+    port = server.connect_address.ip_port
+
+    Addrinfo.define_singleton_method(:getaddrinfo) do |_, _, family, *_|
+      case family
+      when Socket::AF_INET6 then sleep(0.1); raise Socket::ResolutionError
+      when Socket::AF_INET then [Addrinfo.tcp("127.0.0.1", port)]
+      end
+    end
+
+    server.close
+
+    # SystemCallError is a workaround for Windows environment
+    assert_raise(Errno::ECONNREFUSED, SystemCallError) do
+      Socket.tcp("localhost", port)
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_v6_address_passed
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    begin
+      begin
+        # Verify that "localhost" can be resolved to an IPv6 address
+        Socket.getaddrinfo("localhost", 0, Socket::AF_INET6)
+        server = TCPServer.new("::1", 0)
+      rescue Socket::ResolutionError, Errno::EADDRNOTAVAIL # IPv6 is not supported
+        return
+      end
+
+      _, port, = server.addr
+      server_thread = Thread.new { server.accept }
+
+      Addrinfo.define_singleton_method(:getaddrinfo) do |*_|
+        [Addrinfo.tcp("::1", port)]
+      end
+
+      socket = Socket.tcp("::1", port)
+      assert_true(socket.remote_address.ipv6?)
+    ensure
+      server_thread&.value&.close
+      server&.close
+      socket&.close
+    end
+    RUBY
+  end
+
+  def test_tcp_socket_fast_fallback_is_false
+    server = TCPServer.new("127.0.0.1", 0)
+    _, port, = server.addr
+    server_thread = Thread.new { server.accept }
+    socket = Socket.tcp("127.0.0.1", port, fast_fallback: false)
+    assert_true(socket.remote_address.ipv4?)
+  ensure
+    server_thread&.value&.close
+    server&.close
+    socket&.close
+  end
+
+  def test_tcp_fast_fallback
+    opts = %w[-rsocket -W1]
+    assert_separately opts, <<~RUBY
+    assert_true(Socket.tcp_fast_fallback)
+
+    Socket.tcp_fast_fallback = false
+    assert_false(Socket.tcp_fast_fallback)
+
+    Socket.tcp_fast_fallback = true
+    assert_true(Socket.tcp_fast_fallback)
+    RUBY
+  end
 end if defined?(Socket)
