@@ -1,9 +1,10 @@
+# rbs_inline: enabled
 # frozen_string_literal: true
 
 module Lrama
   class Lexer
-    class Token
-      class Ident < Token
+    module Token
+      class Ident < Base
       end
     end
   end
